@@ -106,6 +106,14 @@ ls * 2
 ls - 2
 (ls + 2) %% 3
 ls / ls
+p_ = st_point(0:1)
+ll = st_sfc(ls[[1]], p_)
+ll & st_sfc(p_)
+ll | st_sfc(p_)
+ll %/% st_sfc(p_)
+ll == st_sfc(p_)
+ll != st_sfc(p_)
+
 
 str(x)
 nc = st_read(system.file("shape/nc.shp", package="sf"), quiet = TRUE)
@@ -197,6 +205,9 @@ ls = st_sfc(st_linestring(rbind(c(0,0),c(0,1))),
  st_linestring(rbind(c(2,2),c(2,2.00001))))
 st_sample(ls, 80)
 st_sample(nc[1:2,], size = c(10,20))
+# try with LINES, LongLat, should generate a warning:
+nc[1:2,] %>% st_transform(4326) %>% st_cast("MULTILINESTRING") %>% st_sample(size = c(10,20))
+st_sample(ls, 80, type = "regular")
 
 #class(st_bind_cols(nc, as.data.frame(nc)[1:3]))
 class(dplyr::bind_cols(nc, as.data.frame(nc)[1:3]))
@@ -220,12 +231,15 @@ demo(meuse, ask = FALSE, echo = FALSE)
 st_bbox(meuse)
 library(raster)
 st_bbox(raster(meuse.grid))
+st_bbox(extent(raster()))
 
 # st_to_s2
-x = sf:::st_to_s2(nc)
-x1 = st_geometry(x)
-cc = st_coordinates(x1)
-summary(sqrt(cc[,1]^2+cc[,2]^2+cc[,3]^2))
+if (FALSE) { # stops working with GDAL 2.3.0 / PROJ 5.0.1:
+ x = sf:::st_to_s2(nc)
+ x1 = st_geometry(x)
+ cc = st_coordinates(x1)
+ summary(sqrt(cc[,1]^2+cc[,2]^2+cc[,3]^2))
+}
 
 # check_ring_dir
 m = rbind(c(0,0), c(0,1), c(1,1), c(1,0), c(0,0))
@@ -259,3 +273,25 @@ st_as_sfc(geo, GeoJSON = TRUE)
 st_as_sfc(geo, GeoJSON = TRUE, crs = 4326)
 
 st_as_sfc(st_as_binary(st_sfc(st_point(0:1)))[[1]], crs = 4326)
+
+x = nc
+x$geom = NULL
+class(x)
+
+st_as_sfc(list(st_point(0:1)), crs = 4326)
+
+# crop:
+box = c(xmin = 0, ymin = 0, xmax = 1, ymax = 1)
+
+pol = st_sfc(st_buffer(st_point(c(.5, .5)), .65))
+pol_sf = st_sf(a=1, geom=pol)
+
+st_crop(pol, box)
+st_crop(pol, st_bbox(box))
+st_crop(pol_sf, box)
+st_crop(pol_sf, st_bbox(box))
+
+# new sample methods:
+x = st_sfc(st_polygon(list(rbind(c(0,0),c(90,0),c(90,90),c(0,90),c(0,0))))) # NOT long/lat:
+p <- st_sample(x, 10, type = "regular")
+p <- st_sample(x, 10, type = "hexagonal")
