@@ -34,7 +34,6 @@ test_that("st_transform works", {
 
 test_that("gdal can be loaded, unloaded, and loaded", {
   expect_silent({
-  sf:::.onLoad()
   sf:::.onUnload()
   sf:::.onLoad()
   }
@@ -45,9 +44,17 @@ test_that("st_wrap_dateline works", {
 	expect_silent(x <- st_wrap_dateline(st_sfc(st_linestring(rbind(c(-179,0),c(179,0))), crs = 4326)))
 })
 
+test_that('gdal_subdatasets works', {
+  skip_if_not(sf_extSoftVersion()[["GDAL"]] >= "2.1.0")
+  fname = system.file("nc/cropped.nc", package = "sf")
+  sd2 = gdal_subdatasets(fname)[[2]]
+})
+
 # context("gdal utils")
 test_that('gdal_utils work', {
   skip_on_appveyor() # FIXME:
+  skip_if_not(Sys.getenv("USER") %in% c("edzer", "travis"))
+  skip_if_not(sf_extSoftVersion()[["GDAL"]] >= "2.1.0")
 
   fname = system.file("nc/cropped.nc", package = "sf")
   #fname = system.file("tif/geomatrix.tif", package = "sf")
